@@ -21,7 +21,7 @@ use encoding 'utf8';
 
 sub get_TK_version {
     # Stick with %.2f format - 4.50 vice 4.5
-    return '4.27';
+    return '4.30';
 }
 
 sub get_path {
@@ -52,6 +52,9 @@ sub get_path {
     # The db directory stores virus defs/freshclam-related stuff
     $path->{db} = $path->{clamtk} . '/db';
 
+    # The submit directory stores file submission information
+    $path->{submit} = $path->{clamtk} . '/submit';
+
     # Default variables
     $path->{whitelist_dir} =
         join( ';', $path->{viruses}, '/sys', '/dev', '/proc;' );
@@ -87,7 +90,7 @@ sub def_paths {
     # 1. The .cld files
     # 2. The .cvd files
     # 3. The {daily,main}.info directories
-    # As of 4.23, we're no longer looking foro the .info dirs.
+    # As of 4.23, we're no longer looking for the .info dirs.
     # The .cvd is the compressed database, while .cld is a
     # previous .cvd/.cld with incremental updates.
 
@@ -136,8 +139,10 @@ sub get_AV_version {
     # simple 'clamscan -V'
     # We have to parse this:
     # ClamAV 0.95.3/11220/Fri Jun 18 22:06:39 2010
+    local $ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';
+    delete @ENV{ 'IFS', 'CDPATH', 'ENV', 'BASH_ENV' };
     my $paths = ClamTk::App->get_path('clampath');
-    my ($version) = qx/$paths -V/;
+    my ($version) = qx | $paths -V |;
     $version =~ s/ClamAV\s(.*?)\/.*/$1/;
     chomp($version);
     return $version;
@@ -145,8 +150,12 @@ sub get_AV_version {
 
 sub get_AV_remote {
     # The user may have set the preference to not check this
-    # on startup using 'AVCheck'
-    return 'undef' if ( !ClamTk::Prefs->get_preference('AVCheck') );
+    # on startup using 'AVCheck'.  We'll let the user manually
+    # check anyway if using the update checker, though
+    my $call = ( caller(1) )[0];
+    return 'undef'
+        if ( !ClamTk::Prefs->get_preference('AVCheck')
+        && $call eq 'ClamTk::GUI' );
 
     # Host for version
     my $rmt_host = 'current.cvd.clamav.net';
