@@ -23,15 +23,16 @@ use ClamTk::Results;
 use ClamTk::Update;
 use ClamTk::Schedule;
 use ClamTk::Device;
+use ClamTk::Submit;
 
-use Cwd qw/abs_path/;
-use File::Basename qw/basename dirname/;
-use POSIX qw/locale_h strftime/;
+use Cwd 'abs_path';
+use File::Basename 'basename', 'dirname';
+use POSIX 'locale_h',          'strftime';
+use Date::Calc 'Delta_Days';
+use Digest::MD5 'md5_hex';
 use File::Find::Rule;
-use Date::Calc qw/Delta_Days/;
 use Locale::gettext;
 use File::Copy;
-use Digest::MD5 qw/md5_hex/;
 use LWP::UserAgent;
 
 textdomain('clamtk');
@@ -39,7 +40,7 @@ setlocale( LC_MESSAGES, '' );
 bind_textdomain_codeset( 'clamtk', 'UTF-8' );
 
 use Gtk2;
-use Glib qw/TRUE FALSE/;
+use Glib 'TRUE', 'FALSE';
 
 my $window;
 my $top_label;
@@ -52,7 +53,7 @@ my $pb;          # progressbar
 my $tt;          # tooltips
 my $SCAN;        # scan handle; must be global for stop button
 my $scan_pid;    # pid of scan handle; must be global for stop button
-my $stopped = 0;
+my $stopped = 0; # program is stopped if 1, running if 0
 my $scan_option;
 my $scan_frame;
 my $start_time;
@@ -93,8 +94,10 @@ sub start_gui {
     my $vbox = Gtk2::VBox->new( FALSE, 0 );
     $window->add($vbox);
 
+    #<<< Perltidy can ignore this
     # @entries contain the menubar items
     my @entries = (
+        ### Everything falls under the following 6 rows ###
         [ 'FileMenu',       undef, gettext('_Scan') ],
         [ 'ViewMenu',       undef, gettext('_View') ],
         [ 'OptionsMenu',    undef, gettext('_Options') ],
@@ -102,132 +105,151 @@ sub start_gui {
         [ 'AdvancedMenu',   undef, gettext('_Advanced') ],
         [ 'HelpMenu',       undef, gettext('_Help') ],
 
-        # These fall under "Scan", although it's still called FileMenu
+        ### These fall under "Scan", although it's still called FileMenu. ###
+        # Scan a single file
         [   'Scan_File',        'gtk-copy',
             gettext('A _File'), '<control>F',
-            gettext('Scan a file'), sub { ClamTk::GUI->getfile('file') }
+            gettext('Scan a file'), sub { ClamTk::GUI->getfile('file') },
+	    FALSE
         ],
-        [   'Quick_Home',
-            'gtk-go-down',
-            gettext('Home (_Quick)'),
-            '<control>Q',
+        # Scan home directory, but do not descend
+        [   'Quick_Home', 		'gtk-go-down',
+            gettext('Home (_Quick)'), 	'<control>Q',
             gettext('Quick Home Scan'),
-            sub { ClamTk::GUI->getfile('home') }
+            sub { ClamTk::GUI->getfile('home') },
+	    FALSE
         ],
-        [   'Full_Home',
-            'gtk-goto-bottom',
-            gettext('Home (Recursive)'),
-            '<control>Z',
+        # Scan home directory, descend (recursive)
+        [   'Full_Home', 		'gtk-goto-bottom',
+            gettext('Home (Recursive)'), 	'<control>Z',
             gettext('Full Home Scan'),
-            sub { ClamTk::GUI->getfile('full-home') }
+            sub { ClamTk::GUI->getfile('full-home') },
+	    FALSE
         ],
-        [   'Scan_Directory',
-            'gtk-zoom-in',
-            gettext('A _Directory'),
-            '<control>D',
+        # Scan a single directory
+        [   'Scan_Directory', 		'gtk-directory',
+            gettext('A _Directory'), 	'<control>D',
             gettext('Scan a Directory'),
-            sub { ClamTk::GUI->getfile('dir') }
+            sub { ClamTk::GUI->getfile('dir') },
+	    FALSE
         ],
-        [   'Scan_Device',
-            'gtk-cdrom',
-            gettext('A Device'),
-            '<control>J',
-            gettext('Scan a Device'),
-            sub { ClamTk::Device->look_for_device() }
-        ],
-        [   'Recursive_Scan',
-            'gtk-zoom-fit',
-            gettext('_Recursive Scan'),
-            '<control>R',
+        # Scan a directory, descend (recursive)
+        [   'Recursive_Scan', 		'gtk-sort-ascending',
+            gettext('_Recursive Scan'), '<control>R',
             gettext('Recursively scan a directory'),
-            sub { ClamTk::GUI->getfile('recur') }
+            sub { ClamTk::GUI->getfile('recur') },
+	    FALSE
         ],
+        # Scan a device - relies upon udev and /proc/mounts
+        [   'Scan_Device', 		'gtk-cdrom',
+            gettext('A Device'), 	'<control>J',
+            gettext('Scan a Device'),
+            sub { ClamTk::Device->look_for_device() },
+	    FALSE
+        ],
+        # Quit
         [   'Exit',           'gtk-quit',
             gettext('E_xit'), '<control>X',
-            gettext('Quit this program'), sub { Gtk2->main_quit }
+            gettext('Quit this program'), sub { Gtk2->main_quit },
+	    FALSE
         ],
 
-        # These fall under View
-        [   'ManageHistories',
-            'gtk-index',
-            gettext('Manage _Histories'),
-            "<control>H",
+        ### These fall under View ###
+        # Show dialog of recorded scans
+        [   'ManageHistories', 		'gtk-index',
+            gettext('Manage _Histories'), "<control>H",
             gettext('Select Histories to Delete'),
             sub { history('delete') },
             FALSE
         ],
+        # Closes the scan bar, zeroes out everything
         [   'ClearOutput',                'gtk-clear',
             gettext('Clear _Output'),     '<control>O',
             gettext('Clear the Display'), \&clear_output,
             FALSE
         ],
 
-        # These fall under Quarantine
-        [   'Status', 'gtk-edit', gettext('_Status'), '<control>S',
+        ### These fall under Quarantine ###
+        # Simple window showing number of quarantined objects
+        [   'Status', 			'gtk-edit', 
+	     gettext('_Status'), 	'<control>S',
             gettext('See how many files are quarantined'),
-            \&quarantine_check
+            \&quarantine_check,
+	    FALSE
         ],
-        [   'Maintenance',
-            'gtk-preferences',
-            gettext('_Maintenance'),
-            '<control>M',
+        # Dialog box - delete or return quarantined objects
+        [   'Maintenance', 		'gtk-preferences',
+            gettext('_Maintenance'), 	'<control>M',
             gettext('View files that have been quarantined'),
             \&maintenance,
+	    FALSE
         ],
-        [   'Empty',
-            'gtk-delete',
-            gettext('_Empty Quarantine Folder'),
-            '<control>E',
+        # Delete quarantined objects
+        [   'Empty', 			'gtk-delete',
+            gettext('_Empty Quarantine Folder'), '<control>E',
             gettext('Delete all files that have been quarantined'),
-            \&del_quarantined
+            \&del_quarantined,
+	    FALSE
         ],
 
-        # These fall under Help
-        [   'UpdateSig',
-            'gtk-network',
-            gettext('_Check for updates'),
-            '<control>U',
+        ### These fall under Help ###
+        # Dialog box to update signatures, check for GUI updates
+        [   'UpdateSig', 			'gtk-network',
+            gettext('_Check for updates'), 	'<control>U',
             gettext('Check for updates'),
-            sub { ClamTk::Update->update_dialog() }
+            sub { ClamTk::Update->update_dialog() },
+	    FALSE
         ],
+        # Standard About dialog
         [   'About',                          'gtk-about',
             gettext("_About"),                '<control>A',
-            gettext('About this program...'), \&about
+            gettext('About this program...'), \&about,
+	    FALSE
         ],
 
-        # The final three here are for Advanced
-        [   'Scheduler',
-            'gtk-media-forward',
-            gettext('_Scheduler'),
-            '<control>T',
+        ### The final three here are for Advanced ###
+        # Schedule scans and AV sig updates
+        [   'Scheduler', 		'gtk-media-forward',
+            gettext('_Scheduler'), 	'<control>T',
             gettext('Schedule a daily scan'),
-            sub { ClamTk::Schedule->schedule_dialog() }
+            sub { ClamTk::Schedule->schedule_dialog() },
+	    FALSE
         ],
-        [   "AVsetup",
-            'gtk-refresh',
-            gettext('Rerun AV setup _wizard'),
+        # Change to/from user updates to system
+        [   "AVsetup", 			'gtk-refresh',
+            gettext('Rerun antivirus setup _wizard'),
             '<control>W',
-            gettext('Rerun AV setup wizard'),
-            sub { ClamTk::Update->av_db_select() }
+            gettext('Rerun antivirus setup wizard'),
+            sub { ClamTk::Update->av_db_select() },
+	    FALSE
         ],
+        # Huge preferences window
         [   'Preferences',          'gtk-properties',
             gettext('Preferences'), '<control>P',
             gettext('Preferences'), \&preferences,
-            FALSE
+	    FALSE
+        ],
+	# Submit a file for analysis
+        [   'SubmitFile',          'gtk-convert',
+            gettext('Submit a file for analysis'), '<control>Y',
+            gettext('Submit a file for analysis'),
+	    sub { ClamTk::Submit->analysis() },
+	    FALSE
         ],
     );
-
+    #>>>
     # $ui_info dictates where the options fall
     my $ui_info = "<ui>
         <menubar name='MenuBar'>
          <menu action='FileMenu'>
           <menuitem action='Scan_File'/>
           <menuitem action='Scan_Directory'/>
-          <menuitem action='Scan_Device'/>
           <menuitem action='Recursive_Scan'/>
           <menuitem action='Quick_Home'/>
           <menuitem action='Full_Home'/>
           <separator/>
+          <menuitem action='Scan_Device'/>
+          <separator/>
           <menuitem action='Exit'/>
          </menu>
           <menu action='ViewMenu'>
@@ -243,6 +265,7 @@ sub start_gui {
            <menuitem action='Scheduler'/>
            <menuitem action='AVsetup'/>
            <menuitem action='Preferences'/>
+           <menuitem action='SubmitFile'/>
  	 </menu>
          <menu action='HelpMenu'>
           <menuitem action='UpdateSig'/>
@@ -361,9 +384,9 @@ sub start_gui {
     $status_gui_box->pack_start( $gui_version_text, TRUE, TRUE, 0 );
     $status_table->attach_defaults( $status_gui_box, 0, 3, 1, 2 );
 
-    # Date of virus signatures
+    # Date of antivirus signatures
     my $defs_gui_box = Gtk2::HBox->new( TRUE, 0 );
-    my $defs_text = Gtk2::Label->new( gettext('Virus definitions') );
+    my $defs_text = Gtk2::Label->new( gettext('Antivirus definitions') );
     $defs_version_text = Gtk2::Label->new( gettext('Unknown') );
     $defs_gui_img =
         Gtk2::Image->new_from_stock( 'gtk-apply', 'small-toolbar' );
@@ -377,9 +400,9 @@ sub start_gui {
     my $last_gui_box = Gtk2::HBox->new( TRUE, 0 );
     my $last_gui_img =
         Gtk2::Image->new_from_stock( 'gtk-print-preview', 'small-toolbar' );
-    my $last_text = Gtk2::Label->new( gettext('Last virus scan') );
+    my $last_text = Gtk2::Label->new( gettext('Last scan') );
     $last_date_text = Gtk2::Label->new( gettext('Unknown') );
-    $tt->set_tip( $last_gui_img, gettext('Date of your last virus scan') );
+    $tt->set_tip( $last_gui_img, gettext('Date of your last scan') );
     $last_gui_box->pack_start( $last_gui_img,   TRUE, TRUE, 0 );
     $last_gui_box->pack_start( $last_text,      TRUE, TRUE, 0 );
     $last_gui_box->pack_start( $last_date_text, TRUE, TRUE, 0 );
@@ -457,7 +480,7 @@ sub start_gui {
     $left_status = Gtk2::Label->new( gettext('Files Scanned: ') );
     $bottom_box->pack_start( $left_status, TRUE, TRUE, 5 );
 
-    $mid_status = Gtk2::Label->new( gettext('Viruses Found: ') );
+    $mid_status = Gtk2::Label->new( gettext('Threats Found: ') );
     $bottom_box->pack_start( $mid_status, TRUE, TRUE, 5 );
 
     $right_status = Gtk2::Label->new( gettext('Ready') );
@@ -511,11 +534,14 @@ sub start_gui {
         update_status_frame();
     }
 
-    # quick check for 'crontab'.
-    # probably unnecessary with today's package managers,
-    # but it's easy enough to just disable the Scheduler link
-    # if it doesn't exist
-    my $crontab = qx!which crontab 2>/dev/null!;
+    # Quick check for 'crontab'.
+    # Disable this shortcut if it doesn't exist.
+    # This kind of disabling doesn't disable the Ctrl- shortcut.
+    my $crontab =
+          ( -e '/usr/bin/crontab' )       ? '/usr/bin/crontab'
+        : ( -e '/usr/local/bin/crontab' ) ? '/usr/local/bin/crontab'
+        : ( -e '/bin/crontab' )           ? '/bin/crontab'
+        :                                   '';
     chomp($crontab);
     if ( !$crontab ) {
         warn "crontab not installed - disabling Scheduler\n";
@@ -523,6 +549,37 @@ sub start_gui {
         $del->set_sensitive(FALSE);
     }
 
+    # Quick check for 'udevinfo' or 'udevadm'.
+    # Disable this shortcut if it doesn't exist.
+    # This kind of disabling doesn't disable the Ctrl- shortcut.
+    my $udev;
+    local $ENV{'PATH'} = '/bin:/usr/bin:/sbin';
+    delete @ENV{ 'IFS', 'CDPATH', 'ENV', 'BASH_ENV' };
+    $udev = qx | which udevadm |;
+    chomp($udev);
+
+    if ( !$udev ) {
+        $udev = qx | which udevinfo |;
+        chomp($udev);
+    }
+
+    # If it's still not found, it may
+    # be because /sbin is not in the user's path.
+    if ( !$udev ) {
+        if ( -e '/sbin/udevadm' ) {
+            $udev = '/sbin/udevadm';
+        } elsif ( -e '/sbin/udevinfo' ) {
+            $udev = '/sbin/udevinfo';
+        }
+    }
+
+    if ( !$udev ) {
+        warn "udev not installed - disabling Device scan\n";
+        my $del = $ui->get_widget('/MenuBar/FileMenu/Scan_Device');
+        $del->set_sensitive(FALSE);
+    }
+
+    # Launch!
     Gtk2->main();
 
     return $window;
@@ -535,42 +592,7 @@ sub update_status_frame {
     Gtk2->main_iteration while ( Gtk2->events_pending );
 
     # ClamAV version
-    my ($version) = ClamTk::App->get_AV_version();
-    $engine_version_text->set_text($version);
-
-    # Check for latest version
-    $av_remote = ClamTk::App->get_AV_remote();
-
-    if ( $av_remote eq 'undef' || $av_remote eq 'unavailable' ) {
-        $engine_gui_img->set_from_stock( 'gtk-dialog-question',
-            'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Unable to check') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-    } elsif ( ( $version cmp $av_remote ) == -1 ) {
-        $engine_gui_img->set_from_stock( 'gtk-dialog-error',
-            'small-toolbar' );
-        $tt->set_tip( $engine_gui_img,
-            gettext('The antivirus engine is reporting an outdated engine') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-    } elsif ( ( $version cmp $av_remote ) == 0 ) {
-        $engine_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Current') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-    } elsif ( ( $version cmp $av_remote ) == 1 ) {    # shouldn't happen
-        $engine_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Current') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-    } else {
-        $engine_gui_img->set_from_stock( 'gtk-dialog-question',
-            'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Unable to check') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-    }
+    set_AV_engine_status();
 
     # ClamTk version
     Gtk2->main_iteration while ( Gtk2->events_pending );
@@ -612,6 +634,52 @@ sub update_status_frame {
     return;
 }
 
+sub set_AV_engine_status {
+    my ($version) = ClamTk::App->get_AV_version();
+    $engine_version_text->set_text($version);
+
+    # Check for latest version
+    $av_remote = ClamTk::App->get_AV_remote();
+
+    if ( $av_remote eq 'undef' || $av_remote eq 'unavailable' ) {
+        $engine_gui_img->set_from_stock( 'gtk-dialog-question',
+            'small-toolbar' );
+        $tt->set_tip( $engine_gui_img, gettext('Unable to check') );
+        $window->queue_draw;
+        Gtk2->main_iteration while ( Gtk2->events_pending );
+        return ( gettext('Unable to check') );
+    } elsif ( ( $version cmp $av_remote ) == -1 ) {
+        $engine_gui_img->set_from_stock( 'gtk-dialog-error',
+            'small-toolbar' );
+        $tt->set_tip( $engine_gui_img,
+            gettext('The antivirus engine is reporting an outdated engine') );
+        $window->queue_draw;
+        Gtk2->main_iteration while ( Gtk2->events_pending );
+        return (
+            gettext('The antivirus engine is reporting an outdated engine') );
+    } elsif ( ( $version cmp $av_remote ) == 0 ) {
+        $engine_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
+        $tt->set_tip( $engine_gui_img, gettext('Current') );
+        $window->queue_draw;
+        Gtk2->main_iteration while ( Gtk2->events_pending );
+        return ( gettext('Current') );
+    } elsif ( ( $version cmp $av_remote ) == 1 ) {    # shouldn't happen
+        $engine_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
+        $tt->set_tip( $engine_gui_img, gettext('Current') );
+        $window->queue_draw;
+        Gtk2->main_iteration while ( Gtk2->events_pending );
+        return ( gettext('Current') );
+    } else {
+        $engine_gui_img->set_from_stock( 'gtk-dialog-question',
+            'small-toolbar' );
+        $tt->set_tip( $engine_gui_img, gettext('Unable to check') );
+        $window->queue_draw;
+        Gtk2->main_iteration while ( Gtk2->events_pending );
+        return ( gettext('Unable to check') );
+    }
+    return;
+}
+
 sub set_sig_status {
     if ( !$defs_version_text ) {
 
@@ -643,7 +711,7 @@ sub set_sig_status {
             gettext('Your antivirus signatures are out-of-date') );
     } else {
         $defs_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
-        $tt->set_tip( $defs_gui_img, 'Current' );
+        $tt->set_tip( $defs_gui_img, gettext('Current') );
     }
     return;
 }
@@ -857,6 +925,7 @@ sub getfile {
     if ( $option eq 'file' ) {
         scan(@files);
     } else {
+        # Pass the files to the filter first
         @files = filter( $option, @files );
         # the pulse of the progressbar
         if ( scalar(@files) > 1 ) {
@@ -938,6 +1007,17 @@ sub filter {
     # (unless it's a single file - or option eq 'file')
     my $paths = ClamTk::App->get_path('all');
 
+    # Remove mail directories for now -
+    # until we can parse them... sigh.
+    # http://clamtk.sourceforge.net/faq.html#inbox
+    my @maildirs = qw|
+        .thunderbird	.mozilla-thunderbird
+        .evolution	Mail	kmail|;
+    for my $mailbox (@maildirs) {
+        $mailbox = "$paths->{directory}/$mailbox";
+        @filter = grep { !m#$mailbox# } @filter;
+    }
+
     for my $ignore (
         split(
             /;/,
@@ -967,7 +1047,7 @@ sub scan {
 
     # implicit fork; gives us the PID of clamscan so we can
     # kill it if the user hits the Stop button
-    my $pid = open( $SCAN, "-|", "$command @quoted 2>&1" );
+    my $pid = open( $SCAN, '-|', "$command @quoted 2>&1" );
     defined($pid) or die "couldn't fork: $!\n";
     my $scan_count = 0;
     $scan_status_label->set_text(
@@ -1048,9 +1128,9 @@ sub scan {
         # If we have possible threats, highlight it with bold
         if ( $num_so_far > 0 ) {
             $mid_status->set_markup(
-                sprintf gettext('<b>Viruses Found: %d</b>'), $num_so_far );
+                sprintf gettext('<b>Threats Found: %d</b>'), $num_so_far );
         } else {
-            $mid_status->set_text( sprintf gettext('Viruses Found: %d'),
+            $mid_status->set_text( sprintf gettext('Threats Found: %d'),
                 $num_so_far );
         }
         $num_scanned++;
@@ -1104,7 +1184,7 @@ sub clear_output {
     $scan_status_label->set_text('');
     # Reset the bottom labels
     $left_status->set_text( gettext('Files Scanned: ') );
-    $mid_status->set_text( gettext('Viruses Found: ') );
+    $mid_status->set_text( gettext('Threats Found: ') );
     $right_status->set_text( gettext('Ready') );
     # Hide the scanning frame
     $scan_frame->hide();
@@ -1153,7 +1233,7 @@ sub clean_up {
 
         # sort the directories scanned for display
         my @sorted = sort { $a cmp $b } keys %dirs_scanned;
-        if ( open $REPORT, ">>:encoding(UTF-8)", $virus_log ) {
+        if ( open $REPORT, '>>:encoding(UTF-8)', $virus_log ) {
             print $REPORT "-" x 77, "\n";
             print $REPORT "\nClamTk, v", ClamTk::App->get_TK_version(), "\n",
                 scalar localtime, "\n";
@@ -1165,7 +1245,7 @@ sub clean_up {
             }
             printf $REPORT gettext(
                 "\nFound %d possible %s (%d %s scanned).\n\n"), $num_so_far,
-                $num_so_far == 1 ? gettext('virus') : gettext('viruses'),
+                $num_so_far == 1 ? gettext('threat') : gettext('threats'),
                 $num_scanned,
                 $num_scanned == 1 ? gettext('file') : gettext('files');
         } else {
@@ -1180,14 +1260,14 @@ sub clean_up {
     $left_status->set_text( sprintf gettext('Files Scanned: %d'),
         $num_scanned );
     if ( $num_so_far != 0 ) {
-        $mid_status->set_text( sprintf gettext('Viruses Found: %d'),
+        $mid_status->set_text( sprintf gettext('Threats Found: %d'),
             $num_so_far );
     }
     $right_status->set_text( gettext('Ready') );
     $window->queue_draw;
 
     if ( $num_so_far == 0 ) {
-        print $REPORT gettext("No viruses found.\n") if ($save_log);
+        print $REPORT gettext("No threats found.\n") if ($save_log);
     } else {
         if ($save_log) {
             for my $num ( sort keys %$found ) {
@@ -1660,7 +1740,7 @@ sub view_box {
     $textview->set( editable => FALSE );
 
     my $FILE;    # filehandle for histories log
-    unless ( open( $FILE, "<:encoding(UTF-8)", $full_path ) ) {
+    unless ( open( $FILE, '<:encoding(UTF-8)', $full_path ) ) {
         my $notice = sprintf gettext('Problems opening %s...'), $data->[0];
         show_message_dialog( $window, 'error', 'ok', $notice );
         return;
@@ -1675,9 +1755,9 @@ sub view_box {
         $data->[0];
 
     my $textbuffer = $textview->get_buffer;
-    $textbuffer->create_tag( 'mono', family => 'Monospace' );
+    $textbuffer->create_tag( 'deja', family => 'DejaVu Sans' );
     $textbuffer->insert_with_tags_by_name( $textbuffer->get_start_iter, $text,
-        'mono' );
+        'deja' );
 
     my $scroll_win = Gtk2::ScrolledWindow->new;
     $scroll_win->set_border_width(5);
@@ -1782,7 +1862,7 @@ sub preferences {
     );
 
     my $log_box = Gtk2::CheckButton->new_with_label(
-        gettext('Keep a record of this scan') );
+        gettext('Keep a record of every scan') );
     $log_box->set_active(TRUE) if ( $prefs{SaveToLog} );
     $scan_table->attach_defaults( $log_box, 0, 1, 4, 5 );
     $log_box->signal_connect(
@@ -1945,7 +2025,7 @@ sub preferences {
             my ( $widget, $string, $position ) = @_;
 
             # http://www.ietf.org/rfc/rfc1738.txt
-            if ( $string !~ m#[a-zA-Z0-9\.\-\+/:\@\#]# ) {
+            if ( $string !~ m#[\w\.\-\+/:\@\#]# ) {
                 $ip_box->signal_stop_emission_by_name('insert-text');
             }
             return;
@@ -2024,6 +2104,11 @@ sub preferences {
                     $ip = 'http://' . $ip;
                 }
                 my $port = $port_box->get_value_as_int;
+                if ( $port =~ /^(\d+)$/ ) {
+                    $port = $1;
+                } else {
+                    $port = 8080;
+                }
 
                 # Hate to pull in LWP::UserAgent just for this,
                 # but we need to sanity check it before they get
