@@ -16,12 +16,13 @@ use strict;
 $|++;
 
 use ClamTk::App;
+use ClamTk::Submit;
 
 use encoding 'utf8';
 
-use File::Path;
+use File::Path 'mkpath';
 use Locale::gettext;
-use POSIX qw/locale_h/;
+use POSIX 'locale_h';
 textdomain('clamtk');
 setlocale( LC_MESSAGES, '' );
 bind_textdomain_codeset( 'clamtk', 'UTF-8' );
@@ -30,56 +31,86 @@ sub structure {
     my $paths = ClamTk::App->get_path('all');
 
     # Ensure default paths/files exist.
-    # If they do, ensure they have the proper permissions
+    # If they do, ensure they have the proper permissions.
+    # The default Fedora umask for users is 0002,
+    # while Ubuntu's is 0022... sigh.
+    # I'm going to assume that it's one or the other.
+    # my $umask = sprintf("%04o", umask());
+    my $mask = ( umask() == 2 ) ? '0775' : '0755';
+
+    # This is /home/user/.clamtk/viruses,
+    # used for the quarantine directory
     if ( !-d $paths->{viruses} ) {
-        # This is typically /home/user/.clamtk/viruses,
-        # used for the quarantine directory
-        eval { mkpath( $paths->{viruses}, 0, oct(755) ); };
+        eval { mkpath( $paths->{viruses}, { mode => oct($mask) } ) };
+        warn $@  if ($@);
         return 0 if ($@);
     } else {
         # Ensure the permissions are correct
-        chmod oct(755), $paths->{viruses};
+        chmod oct($mask), $paths->{viruses};
     }
+
+    # This is /home/user/.clamtk/history,
+    # which holds records of scans
     if ( !-d $paths->{history} ) {
-        # This is typically /home/user/.clamtk/history,
-        # which holds records of scans
-        eval { mkpath( $paths->{history}, 0, oct(755) ); };
+        eval { mkpath( $paths->{history}, { mode => oct($mask) } ) };
+        warn $@  if ($@);
         return 0 if ($@);
     } else {
         # Ensure the permissions are correct
-        chmod oct(755), $paths->{history};
+        chmod oct($mask), $paths->{history};
     }
+
+    # The path /home/user/.clamtk/db stores signatures
     if ( !-d $paths->{db} ) {
-        # This is typically /home/user/.clamtk/db
-        eval { mkpath( $paths->{db}, 0, oct(755) ); };
+        eval { mkpath( $paths->{db}, { mode => oct($mask) } ) };
         warn $@  if $@;
         return 0 if ($@);
     } else {
         # Ensure the permissions are correct
-        chmod oct(755), $paths->{db};
+        chmod oct($mask), $paths->{db};
     }
+
+    # This is /home/user/.clamtk/prefs,
+    # a custom INI-style file
     if ( !-e $paths->{prefs} ) {
-        # This is /home/user/.clamtk/prefs,
-        # a custom INI-style file
-        open( my $F, ">:encoding(UTF-8)", $paths->{prefs} )
+        open( my $F, '>:encoding(UTF-8)', $paths->{prefs} )
             or do {
             warn "Unable to create preferences! $!\n";
             return 0;
             };
         close($F);
         eval { custom_prefs() };
+        warn $@  if ($@);
         return 0 if ($@);
     }
+
+    # This is /home/user/.clamtk/restore, which holds
+    # information for putting back false positives
     if ( !-e $paths->{restore} ) {
-        # This is /home/user/.clamtk/restore, which holds
-        # information for putting back false positives
-        open( my $F, ">:encoding(UTF-8)", $paths->{restore} )
+        open( my $F, '>:encoding(UTF-8)', $paths->{restore} )
             or do {
             warn "Unable to create restore file! $!\n";
             return 0;
             };
         close($F);
     }
+
+    # This is /home/user/.clamtk/submit
+    if ( !-d $paths->{submit} ) {
+        eval { mkpath( $paths->{submit}, { mode => oct($mask) } ) };
+        warn $@  if $@;
+        return 0 if ($@);
+    } else {
+        # Ensure the permissions are correct
+        chmod oct($mask), $paths->{submit};
+    }
+
+    # Now create the file to track submissions (ClamTk::Submit).
+    # Not going to recreate everything here.
+    if ( !-e "$paths->{submit}/times" ) {
+        ClamTk::Submit->create_file();
+    }
+
     return 1;
 }
 
@@ -89,7 +120,7 @@ sub custom_prefs {
     # Get the user's current prefs
     my $paths = ClamTk::App->get_path('prefs');
 
-    open( my $F, "<:encoding(UTF-8)", $paths )
+    open( my $F, '<:encoding(UTF-8)', $paths )
         or do {
         warn "Unable to read preferences! $!\n";
         return 0;
@@ -152,7 +183,7 @@ sub get_all_prefs {
     # the preferences rather than just one.
     my %pkg;
     my $paths = ClamTk::App->get_path('prefs');
-    open( my $F, "<:encoding(UTF-8)", $paths )
+    open( my $F, '<:encoding(UTF-8)', $paths )
         or do {
         warn "Unable to read preferences! $!\n";
         return 0;
@@ -171,7 +202,7 @@ sub write_all {
     my %loc = @_;
 
     my $paths = ClamTk::App->get_path('prefs');
-    open( my $F, ">:encoding(UTF-8)", $paths )
+    open( my $F, '>:encoding(UTF-8)', $paths )
         or do {
         warn "Unable to write preferences! $!\n";
         return 0;
@@ -189,7 +220,7 @@ sub set_preference {
     my ( undef, $wk, $wv ) = @_;    # undef = package name
     my $paths = ClamTk::App->get_path('prefs');
 
-    open( my $F, "<:encoding(UTF-8)", $paths )
+    open( my $F, '<:encoding(UTF-8)', $paths )
         or do {
         warn "Unable to read preferences! $!\n";
         return 0;
@@ -203,7 +234,7 @@ sub set_preference {
     }
     close($F);
 
-    open( $F, ">:encoding(UTF-8)", $paths )
+    open( $F, '>:encoding(UTF-8)', $paths )
         or return -1;
 
     while ( my ( $k, $v ) = each %pkg ) {
@@ -220,7 +251,7 @@ sub get_preference {
 
     my $paths = ClamTk::App->get_path('prefs');
     my %pkg;
-    open( my $F, "<:encoding(UTF-8)", $paths )
+    open( my $F, '<:encoding(UTF-8)', $paths )
         or do {
         warn "Unable to read preferences! $!\n";
         return 0;
@@ -269,7 +300,7 @@ sub restore {
     my %p;
     my $restore_path = ClamTk::App->get_path('restore');
 
-    open( my $F, "<:encoding(UTF-8)", $restore_path ) or do {
+    open( my $F, '<:encoding(UTF-8)', $restore_path ) or do {
         warn "Can't open restore file for reading: $!\n";
         return -1;
     };
@@ -300,7 +331,7 @@ sub restore {
             #warn "File $wanted already exists?\n";
             return -1;
         }
-        open( $F, ">:encoding(UTF-8)", $restore_path ) or do {
+        open( $F, '>:encoding(UTF-8)', $restore_path ) or do {
             warn "Can't open restore file for writing: $!\n";
             return -1;
         };
@@ -315,7 +346,7 @@ sub restore {
     }
 
     if ( $job eq 'remove' ) {
-        open( $F, ">:encoding(UTF-8)", $restore_path ) or do {
+        open( $F, '>:encoding(UTF-8)', $restore_path ) or do {
             warn "Can't open restore file for writing: $!\n";
             return -1;
         };
@@ -359,7 +390,7 @@ sub restore_file_fix {
 
     # Now grab the md5sums the restore file says we have
     my %q;
-    open( my $F, "<:encoding(UTF-8)", $restore )
+    open( my $F, '<:encoding(UTF-8)', $restore )
         or return;
     while (<$F>) {
         chomp;
