@@ -18,20 +18,23 @@ use strict;
 #use warnings;    # disabled upon release
 $|++;
 
-use POSIX qw/locale_h strftime/;
-use Locale::gettext;
+use encoding 'utf8';
 
+use POSIX 'locale_h';
+use Locale::gettext;
 textdomain("clamtk");
 setlocale( LC_MESSAGES, "" );
 bind_textdomain_codeset( "clamtk", "UTF-8" );
 
 use Gtk2;
-use Glib qw/TRUE FALSE/;
-
-use encoding 'utf8';
+use Glib 'TRUE', 'FALSE';
 
 # This should be under /usr/bin, but we'll check anyway.
-my $cmd = qx!which crontab 2>/dev/null!;
+my $cmd =
+      ( -e '/usr/bin/crontab' )       ? '/usr/bin/crontab'
+    : ( -e '/usr/local/bin/crontab' ) ? '/usr/local/bin/crontab'
+    : ( -e '/bin/crontab' )           ? '/bin/crontab'
+    :                                   '';
 chomp($cmd);
 
 my $window;
@@ -414,9 +417,11 @@ sub apply_defs {
     }
 
     # Add config file if user has configured a proxy
-    if ( ClamTk::Prefs->get_preference('HTTPProxy') == 2 ) {
-        if ( -e "$paths->{db}/local.conf" ) {
-            $full_cmd .= " --config-file=$paths->{db}/local.conf";
+    if ( ClamTk::Prefs->get_preference('HTTPProxy') ) {
+        if ( ClamTk::Prefs->get_preference('HTTPProxy') == 2 ) {
+            if ( -e "$paths->{db}/local.conf" ) {
+                $full_cmd .= " --config-file=$paths->{db}/local.conf";
+            }
         }
     }
 
