@@ -22,13 +22,13 @@ use encoding 'utf8';
 use ClamTk::GUI;
 
 use Locale::gettext;
-use POSIX qw/locale_h/;
+use POSIX 'locale_h';
 textdomain('clamtk');
 setlocale( LC_MESSAGES, '' );
 bind_textdomain_codeset( 'clamtk', 'UTF-8' );
 
 use Gtk2;
-use Glib qw/TRUE FALSE/;
+use Glib 'TRUE', 'FALSE';
 
 my ( $win, $update_list );
 my ( $go_btn, $close_btn, $cancel_btn );
@@ -77,6 +77,8 @@ sub update_dialog {
 
     my $paths = ClamTk::App->get_path('all');
 
+    # Add signature updates as an option IF
+    # the user is root, or has opted to do it manually
     if ( $> == 0 || $user_can ) {
         if ( $paths->{freshclam} ) {
             push @{ $update_list->{data} },
@@ -88,6 +90,7 @@ sub update_dialog {
         }
     }
 
+    # Add option to check for GUI updates
     push @{ $update_list->{data} },
         [
         gettext('GUI updates'),
@@ -95,11 +98,22 @@ sub update_dialog {
         $gui_box, gettext('N/A'),
         ];
 
+    # Add option to check for ClamAV engine updates
+    push @{ $update_list->{data} },
+        [
+        gettext('Antivirus engine'),
+        gettext('Check for antivirus engine updates'),
+        $gui_box, gettext('N/A'),
+        ];
+
+    # This box will hold buttons to check for updates, cancel,
+    # close the window, and if the user's root, warn them.
     my $hbox = Gtk2::HButtonBox->new();
     $win->vbox->pack_start( $hbox, FALSE, FALSE, 0 );
     $hbox->set_layout_default('spread');
     $win->vbox->set_focus_child($hbox);
 
+    # Check for updates button
     my $go_img = Gtk2::Image->new_from_stock( 'gtk-redo', 'small-toolbar' );
     $go_btn = Gtk2::Button->new();
     $go_btn->signal_connect( clicked => \&decision );
@@ -108,6 +122,7 @@ sub update_dialog {
     $tt->set_tip( $go_btn, gettext('Check for updates') );
     $hbox->add($go_btn);
 
+    # Close window button
     $close_btn = Gtk2::Button->new_from_stock('gtk-close');
     $hbox->add($close_btn);
     $close_btn->signal_connect(
@@ -118,6 +133,7 @@ sub update_dialog {
     );
     $tt->set_tip( $close_btn, gettext('Close this window') );
 
+    # Cancel button
     $cancel_btn = Gtk2::Button->new_from_stock('gtk-cancel');
     $hbox->add($cancel_btn);
     $cancel_btn->signal_connect(
@@ -134,6 +150,7 @@ sub update_dialog {
     );
     $cancel_btn->set_sensitive(FALSE);
 
+    # Warn the user not to do things as root
     if ( $> == 0 ) {
         my $warning_btn = Gtk2::Button->new_from_stock('gtk-dialog-warning');
         $hbox->add($warning_btn);
@@ -162,20 +179,21 @@ sub update_dialog {
     # This should only be expensive the first time or if the user
     # has automatic (cron) updates scheduled and doesn't come here often.
     if ( $> != 0 && ClamTk::Prefs->get_preference('TruncateLog') ) {
-        if ( open( my $f, '<', "$paths->{db}/freshclam.log" ) ) {
-            my @log = ();
+        my $log = "$paths->{db}/freshclam.log";
+        if ( open( my $f, '<', $log ) ) {
+            my $tmp_file = "$paths->{db}/temp.tmp";
+            my @log      = ();
             @log = <$f>;
             close($f);
             if ( scalar(@log) > 30 ) {
-                open( my $t, '>', "$paths->{db}/temp.tmp" )
+                open( my $t, '>', $tmp_file )
                     or last;
                 for ( -30 .. 0 ) {
                     print $t $log[$_];
                 }
                 close($t);
-                rename( "$paths->{db}/temp.tmp",
-                    "$paths->{db}/freshclam.log" );
-                unlink("$paths->{db}/temp.tmp");
+                rename( $tmp_file, $log );
+                unlink($tmp_file);
             }
         }
     }
@@ -196,10 +214,12 @@ sub decision {
     $cancel_btn->set_sensitive(TRUE);
     $win->queue_draw;
 
+    # Go through the rows of options
     for my $row ( 0 .. $scalar - 1 ) {
         Gtk2->main_iteration while Gtk2->events_pending;
         $rowref = $update_list->{data}[$row];
-        if ( $rowref->[2] == 1 ) {    # is it enabled?
+        # Find out if that option is enabled
+        if ( $rowref->[2] == 1 ) {
             if ( $rowref->[0] eq gettext('Signature updates') ) {
                 $update_list->{data}[$row][3] = gettext('Checking...');
                 $value = update_signatures($row);
@@ -220,6 +240,10 @@ sub decision {
                     : $value == 5 ? gettext('Check failed')
                     :               '';
                 ClamTk::GUI->set_tk_status($value);
+            } elsif ( $rowref->[0] eq gettext('Antivirus engine') ) {
+                $update_list->{data}[$row][3] = gettext('Checking...');
+                $value = ClamTk::GUI->set_AV_engine_status();
+                $update_list->{data}[$row][3] = $value;
             } else {
                 #warn 'ref = ', $rowref->[0], "\n";
             }
@@ -244,10 +268,14 @@ sub update_signatures {
     my $paths = ClamTk::App->get_path('all');
 
     my $command = $paths->{freshclam};
+    # If the user will update the signatures manually,
+    # append the appropriate paths
     if ( ClamTk::Prefs->get_preference('Update') eq 'single' ) {
         $command
             .= " --datadir=$paths->{db} --log=$paths->{db}/freshclam.log";
     }
+
+    # Did the user set the proxy option?
     if ( ClamTk::Prefs->get_preference('HTTPProxy') ) {
         if ( ClamTk::Prefs->get_preference('HTTPProxy') == 2 ) {
             if ( -e "$paths->{db}/local.conf" ) {
@@ -404,7 +432,7 @@ sub av_db_select {
     my $choice = Gtk2::Dialog->new();
     $choice->signal_connect( destroy => sub { $choice->destroy; } );
     $choice->set_title( gettext('Antivirus Signatures') );
-    $choice->set_default_size( 325, 230 );
+    $choice->set_default_size( 375, 240 );
 
     if ( -e '/usr/share/pixmaps/clamtk.png' ) {
         $choice->set_default_icon_from_file('/usr/share/pixmaps/clamtk.png');
@@ -424,12 +452,19 @@ sub av_db_select {
     $sw->set_border_width(5);
 
     # This is horrible and will be changed in 4.5 or 5.0.
-    my $dialog =
-        gettext( "\nPlease choose how you will update your antivirus "
-            . "signatures.\n\nIf this is a multi-user system or you have an "
-            . "administrator, you should probably choose 'System Wide'.\n\n"
-            . "If you need to be able to update the signatures yourself, "
-            . "you should probably choose 'Single User'.\n" );
+    my $dialog;
+    $dialog .= gettext(
+        'Please choose how you will update your antivirus signatures.');
+    $dialog .= "\n\n";
+    $dialog
+        .= gettext( 'If you would like to update the signatures yourself, '
+            . 'choose Manual.' );
+    $dialog .= "\n\n";
+    $dialog
+        .= gettext(
+        'If your computer automatically receives updates, choose Automatic.'
+        );
+    $dialog .= "\n";
 
     my $buffer = $view->get_buffer;
     $buffer->create_tag( 'mono', family => 'Monospace' );
@@ -454,7 +489,7 @@ sub av_db_select {
     $cancel_btn->signal_connect( clicked => sub { $choice->destroy } );
 
     # 'Single' button - user will update
-    $single_btn = Gtk2::Button->new( gettext('Single User') );
+    $single_btn = Gtk2::Button->new( gettext('Manual') );
     $c_box->add($single_btn);
     $single_btn->signal_connect(
         clicked => sub {
@@ -466,7 +501,7 @@ sub av_db_select {
     );
 
     # 'Shared' button - system updates
-    $sys_btn = Gtk2::Button->new( gettext('System Wide') );
+    $sys_btn = Gtk2::Button->new( gettext('Automatic') );
     $c_box->add($sys_btn);
     $sys_btn->signal_connect(
         clicked => sub {
